@@ -1,20 +1,7 @@
 /*
-   Copyright (C) 2015-2020 Laurent Montel <montel@kde.org>
+   SPDX-FileCopyrightText: 2015-2020 Laurent Montel <montel@kde.org>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+   SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include "gravatarresolvurljob.h"
@@ -24,7 +11,6 @@
 #include <PimCommon/NetworkManager>
 
 #include <QCryptographicHash>
-#include <QPixmap>
 #include <QUrlQuery>
 #include <QNetworkConfigurationManager>
 
@@ -45,6 +31,7 @@ public:
 
     enum Backend {
         None = 0x0,
+        Libravatar = 0x1,
         Gravatar = 0x2
     };
     int mBackends = Gravatar;
@@ -67,15 +54,16 @@ GravatarResolvUrlJob::~GravatarResolvUrlJob()
 bool GravatarResolvUrlJob::canStart() const
 {
     if (PimCommon::NetworkManager::self()->networkConfigureManager()->isOnline()) {
+        //qCDebug(GRAVATAR_LOG) << "email " << d->mEmail;
         return !d->mEmail.trimmed().isEmpty() && (d->mEmail.contains(QLatin1Char('@')));
     } else {
         return false;
     }
 }
 
-QUrl GravatarResolvUrlJob::generateGravatarUrl()
+QUrl GravatarResolvUrlJob::generateGravatarUrl(bool useLibravatar)
 {
-    return createUrl();
+    return createUrl(useLibravatar);
 }
 
 bool GravatarResolvUrlJob::hasGravatar() const
@@ -91,8 +79,17 @@ void GravatarResolvUrlJob::startNetworkManager(const QUrl &url)
         d->mNetworkAccessManager->setStrictTransportSecurityEnabled(true);
         d->mNetworkAccessManager->enableStrictTransportSecurityStore(true);
         connect(d->mNetworkAccessManager, &QNetworkAccessManager::finished, this, &GravatarResolvUrlJob::slotFinishLoadPixmap);
+
     }
-    d->mNetworkAccessManager->get(QNetworkRequest(url));
+
+    QNetworkRequest req(url);
+    req.setAttribute(QNetworkRequest::HttpPipeliningAllowedAttribute, true);
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
+    req.setAttribute(QNetworkRequest::HTTP2AllowedAttribute, true);
+#else
+    req.setAttribute(QNetworkRequest::Http2AllowedAttribute, true);
+#endif
+    d->mNetworkAccessManager->get(req);
 }
 
 void GravatarResolvUrlJob::start()
@@ -119,11 +116,15 @@ void GravatarResolvUrlJob::processNextBackend()
     }
 
     QUrl url;
-    if (d->mBackends & GravatarResolvUrlJobPrivate::Gravatar) {
+    if (d->mBackends & GravatarResolvUrlJobPrivate::Libravatar) {
+        d->mBackends &= ~GravatarResolvUrlJobPrivate::Libravatar;
+        url = createUrl(true);
+    } else if (d->mBackends & GravatarResolvUrlJobPrivate::Gravatar) {
         d->mBackends &= ~GravatarResolvUrlJobPrivate::Gravatar;
-        url = createUrl();
+        url = createUrl(false);
     }
 
+    //qDebug() << " url " << url;
     Q_EMIT resolvUrl(url);
     if (!cacheLookup(d->mCalculatedHash)) {
         startNetworkManager(url);
@@ -134,8 +135,9 @@ void GravatarResolvUrlJob::processNextBackend()
 
 void GravatarResolvUrlJob::slotFinishLoadPixmap(QNetworkReply *reply)
 {
-    if (reply->error() != QNetworkReply::NoError) {
-        d->mPixmap.loadFromData(reply->readAll());
+    if (reply->error() == QNetworkReply::NoError) {
+        const QByteArray data = reply->readAll();
+        d->mPixmap.loadFromData(data);
         d->mHasGravatar = true;
         //For the moment don't use cache other we will store a lot of pixmap
         if (!d->mUseDefaultPixmap) {
@@ -169,6 +171,32 @@ Hash GravatarResolvUrlJob::calculateHash()
     return Hash(QCryptographicHash::hash(email, QCryptographicHash::Md5), Hash::Md5);
 }
 
+bool GravatarResolvUrlJob::fallbackGravatar() const
+{
+    return d->mBackends & GravatarResolvUrlJobPrivate::Gravatar;
+}
+
+void GravatarResolvUrlJob::setFallbackGravatar(bool fallbackGravatar)
+{
+    if (fallbackGravatar)
+        d->mBackends |= GravatarResolvUrlJobPrivate::Gravatar;
+    else
+        d->mBackends &= ~GravatarResolvUrlJobPrivate::Gravatar;
+}
+
+bool GravatarResolvUrlJob::useLibravatar() const
+{
+    return d->mBackends & GravatarResolvUrlJobPrivate::Libravatar;
+}
+
+void GravatarResolvUrlJob::setUseLibravatar(bool useLibravatar)
+{
+    if (useLibravatar)
+        d->mBackends |= GravatarResolvUrlJobPrivate::Libravatar;
+    else
+        d->mBackends &= ~GravatarResolvUrlJobPrivate::Libravatar;
+}
+
 bool GravatarResolvUrlJob::useDefaultPixmap() const
 {
     return d->mUseDefaultPixmap;
@@ -204,7 +232,7 @@ Hash GravatarResolvUrlJob::calculatedHash() const
     return d->mCalculatedHash;
 }
 
-QUrl GravatarResolvUrlJob::createUrl()
+QUrl GravatarResolvUrlJob::createUrl(bool useLibravatar)
 {
     QUrl url;
     d->mCalculatedHash = Hash();
@@ -220,8 +248,11 @@ QUrl GravatarResolvUrlJob::createUrl()
         query.addQueryItem(QStringLiteral("s"), QString::number(d->mSize));
     }
     url.setScheme(QStringLiteral("https"));
-    url.setHost(QStringLiteral("secure.gravatar.com"));
-    url.setPort(443);
+    if (useLibravatar) {
+        url.setHost(QStringLiteral("seccdn.libravatar.org"));
+    } else {
+        url.setHost(QStringLiteral("secure.gravatar.com"));
+    }
     d->mCalculatedHash = calculateHash();
     url.setPath(QLatin1String("/avatar/") + d->mCalculatedHash.hexString());
     url.setQuery(query);
